@@ -1,9 +1,9 @@
 import concurrent.futures
+import contextlib
 from pathlib import Path
 import re
 import stat
 
-import rpm
 from rpmlint.arparser import ArParser
 from rpmlint.checks.AbstractCheck import AbstractCheck
 from rpmlint.lddparser import LddParser
@@ -12,14 +12,16 @@ from rpmlint.pkg import FakePkg, InstalledPkg
 from rpmlint.readelfparser import ReadelfParser
 from rpmlint.stringsparser import StringsParser
 
+KERNEL_MODULES_PATHS = ('/lib/modules/', '/usr/lib/modules/')
+GLIBC_EMPTY_ARCHIVES = ('libanl', 'libdl', 'libpthread', 'librt', 'libutil')
+
 
 class BinariesCheck(AbstractCheck):
     """
     Checks for binary files in the package.
     """
-    srcname_regex = re.compile(r'(.*?)-[0-9]')
     validso_regex = re.compile(r'(\.so\.\d+(\.\d+)*|\d\.so)$')
-    soversion_regex = re.compile(r'.*?([0-9][.0-9]*)\.so|.*\.so\.([0-9][.0-9]*).*')
+    soversion_regex = re.compile(r'.*?(-(?P<pkgversion>[0-9][.0-9]*))?\.so(\.(?P<soversion>[0-9][.0-9]*))?')
     usr_lib_regex = re.compile(r'^/usr/lib(64)?/')
     ldso_soname_regex = re.compile(r'^ld(-linux(-(ia|x86_)64))?\.so')
 
@@ -30,11 +32,25 @@ class BinariesCheck(AbstractCheck):
     la_file_regex = re.compile(r'\.la$')
     invalid_dir_ref_regex = re.compile(r'/(home|tmp)(\W|$)')
     usr_arch_share_regex = re.compile(r'/share/.*/(?:x86|i.86|x86_64|ppc|ppc64|s390|s390x|ia64|m68k|arm|aarch64|mips|riscv)')
+    python_module_regex = re.compile(r'.*\.(\w*(python|pypy)\w*(-\w+){4}|abi3)\.so')
+    # Starts with ELF or with a special qualifiers like setuid or setgid
+    # https://github.com/rpm-software-management/rpmlint/issues/1088
+    elf_regex = re.compile(r'^(\w+ )?ELF ')
+
+    lto_text_like_sections = {'.preinit_array', '.init_array', '.fini_array'}
+    # The following sections are part of the RX ABI and do correspond to .text, .data and .bss
+    lto_text_like_sections |= {'P', 'D_1', 'B_1'}
+
+    # The list is taken from glibc: sysdeps/${arch}/stackinfo.h
+    default_executable_stack_archs = re.compile(r'alpha|arm.*|hppa|i.86|m68k|microblaze|mips|ppc|s390|s390x|sh|sparc|x86_64')
+
+    rpath_origin = '$ORIGIN'
+    hpc_locations = ('/usr/lib/mpi/', '/usr/lib64/mpi/', '/usr/lib/hpc/')
 
     def __init__(self, config, output):
         super().__init__(config, output)
         self.checked_files = 0
-        self.system_lib_paths = config.configuration['SystemLibPaths']
+        self.system_lib_paths = tuple(config.configuration['SystemLibPaths'])
         self.pie_exec_regex_list = []
         for regex in config.configuration['PieExecutables']:
             self.pie_exec_regex_list.append(re.compile(regex))
@@ -60,7 +76,12 @@ class BinariesCheck(AbstractCheck):
                                 self._check_library_dependency,
                                 self._check_forbidden_functions,
                                 self._check_executable_shlib,
-                                self._check_optflags]
+                                self._check_optflags,
+                                self._check_hash_sections,
+                                self._check_no_patchable_function_entries_in_archive]
+
+    def reset(self):
+        self.checked_files = 0
 
     @staticmethod
     def create_nonlibc_regexp_call(call):
@@ -79,11 +100,8 @@ class BinariesCheck(AbstractCheck):
         """
         if 'shell script' in pkgfile.magic:
             file_start = None
-            try:
-                with open(pkgfile.path, 'rb') as inputf:
-                    file_start = inputf.read(2048)
-            except IOError:
-                pass
+            with contextlib.suppress(IOError), open(pkgfile.path, 'rb') as inputf:
+                file_start = inputf.read(2048)
             if (file_start and b'This wrapper script should never '
                                b'be moved out of the build directory'
                     in file_start):
@@ -98,10 +116,10 @@ class BinariesCheck(AbstractCheck):
         If so then print a corresponding error with the matching line numbers.
         """
         if self.la_file_regex.search(fname):
-            lines = pkg.grep(self.invalid_dir_ref_regex, fname)
-            if lines:
+            line = pkg.grep(self.invalid_dir_ref_regex, fname)
+            if line:
                 self.output.add_info('E', pkg, 'invalid-la-file', fname,
-                                     '(line %s)' % ', '.join(lines))
+                                     f'(line {line})')
 
     def _check_binary_in_noarch(self, pkg, bin_name):
         """
@@ -153,7 +171,7 @@ class BinariesCheck(AbstractCheck):
         executable.
         """
         if not self.is_shobj and not self.is_pie_exec:
-            if any(regex.search(bin_name) for regex in self.pie_exec_regex_list):
+            if any(regex.fullmatch(bin_name) for regex in self.pie_exec_regex_list):
                 self.output.add_info('E', pkg,
                                      'non-position-independent-executable',
                                      bin_name)
@@ -187,13 +205,13 @@ class BinariesCheck(AbstractCheck):
                         not self.versioned_dir_regex.search(fn):
                     self.output.add_info('E', pkg, 'non-versioned-file-in-library-package', f)
 
-    def _check_no_binary(self, pkg, has_binary, multi_pkg, has_file_in_lib64):
+    def _check_no_binary(self, pkg, has_binary, has_file_in_lib64):
         """
         Check if the arch dependent package contains any binaries.
 
         Print an error if there is no binary and it's not noarch.
         """
-        if not has_binary and not multi_pkg and not has_file_in_lib64 and \
+        if not has_binary and not has_file_in_lib64 and \
                 pkg.arch != 'noarch':
             self.output.add_info('E', pkg, 'no-binary')
 
@@ -216,7 +234,7 @@ class BinariesCheck(AbstractCheck):
         if has_usr_lib_file and not has_binary_in_usr_lib:
             self.output.add_info('W', pkg, 'only-non-binary-in-usr-lib')
 
-    def _check_no_text_in_archive(self, pkg, pkgfile_path, path):
+    def _check_no_text_in_archive(self, pkg, pkgfile):
         """
         For an archive, test if any .text sections is non-empty.
         """
@@ -225,18 +243,37 @@ class BinariesCheck(AbstractCheck):
                 if comment.startswith('GHC '):
                     return
 
+            # Starting with glibc 2.34, some static libraries were moved to libc
+            # and there are empty archives for backward compatibility. Skip these
+            # libraries.
+            stem = Path(pkgfile.name).stem
+            if stem in GLIBC_EMPTY_ARCHIVES or (stem.endswith('_p') and stem[:-2] in GLIBC_EMPTY_ARCHIVES):
+                return
+
             for elf_file in self.readelf_parser.section_info.elf_files:
                 for section in elf_file:
                     sn = section.name
-                    if ((sn == '.preinit_array' or sn == '.init_array' or
+                    if ((sn in self.lto_text_like_sections or
                          sn == '.fini_array' or sn.startswith('.text') or
                          sn.startswith('.data')) and
                             section.size > 0):
                         return
-            self.output.add_info('E', pkg, 'lto-no-text-in-archive', path)
+            self.output.add_info('E', pkg, 'lto-no-text-in-archive', pkgfile.name)
             return
 
-    def _check_missing_symtab_in_archive(self, pkg, pkgfile_path, path):
+    def _check_no_patchable_function_entries_in_archive(self, pkg, pkgfile):
+        """
+        For static libraries, we should not ship __patchable_function_entries as it can
+        accidentally make a shared library or an executable live-patchable.
+        """
+        if self.is_archive:
+            for elf_file in self.readelf_parser.section_info.elf_files:
+                for section in elf_file:
+                    if section.name == '__patchable_function_entries':
+                        self.output.add_info('E', pkg, 'patchable-function-entry-in-archive', pkgfile.name)
+                        return
+
+    def _check_missing_symtab_in_archive(self, pkg, pkgfile):
         """
         FIXME Add test coverage.
         """
@@ -246,38 +283,40 @@ class BinariesCheck(AbstractCheck):
                     if section.name == '.symtab':
                         return
 
-            self.output.add_info('E', pkg, 'static-library-without-symtab', path)
+            self.output.add_info('E', pkg, 'static-library-without-symtab', pkgfile.name)
 
-    def _check_missing_debug_info_in_archive(self, pkg, pkgfile_path, path):
+    def _check_missing_debug_info_in_archive(self, pkg, pkgfile):
         if self.is_archive:
             for elf_file in self.readelf_parser.section_info.elf_files:
                 for section in elf_file:
                     if section.name.startswith('.debug_'):
                         return
-            self.output.add_info('E', pkg, 'static-library-without-debuginfo', path)
+            self.output.add_info('E', pkg, 'static-library-without-debuginfo', pkgfile.name)
 
     # Check for LTO sections
-    def _check_lto_section(self, pkg, pkgfile_path, path):
+    def _check_lto_section(self, pkg, pkgfile):
         for elf_file in self.readelf_parser.section_info.elf_files:
             for section in elf_file:
                 if '.gnu.lto_.' in section.name:
-                    self.output.add_info('E', pkg, 'lto-bytecode', path)
+                    self.output.add_info('E', pkg, 'lto-bytecode', pkgfile.name)
                     return
 
-    def _check_executable_stack(self, pkg, pkgfile_path, path):
+    def _check_executable_stack(self, pkg, pkgfile):
         """
         Check if the stack is declared as executable which is usually an error.
-
-        FIXME Add test coverage.
         """
 
+        # Skip architectures that have non-executable stack by default
+        if pkg.arch and not self.default_executable_stack_archs.fullmatch(pkg.arch):
+            return
+
         # Do not check kernel modules and archives
-        if not self.is_archive and not path.startswith('/lib/modules/'):
+        if not self.is_archive and not any(pkgfile.name.startswith(p) for p in KERNEL_MODULES_PATHS):
             stack_headers = [h for h in self.readelf_parser.program_header_info.headers if h.name == 'GNU_STACK']
             if not stack_headers:
-                self.output.add_info('E', pkg, 'missing-PT_GNU_STACK-section', path)
+                self.output.add_info('E', pkg, 'missing-PT_GNU_STACK-section', pkgfile.name)
             elif 'E' in stack_headers[0].flags:
-                self.output.add_info('E', pkg, 'executable-stack', path)
+                self.output.add_info('E', pkg, 'executable-stack', pkgfile.name)
 
     def _check_soname_symlink(self, pkg, shlib, soname):
         """
@@ -291,14 +330,14 @@ class BinariesCheck(AbstractCheck):
         try:
 
             link = pkg.files[str(symlink)].linkto
-            if link not in (shlib, path.parent, ''):
+            if link not in (shlib, path.name, ''):
                 self.output.add_info('E', pkg, 'invalid-ldconfig-symlink', shlib, link)
         except KeyError:
             # if we do not have a symlink, report an issue
             if path.name.startswith('lib') or path.name.startswith('ld-'):
                 self.output.add_info('E', pkg, 'no-ldconfig-symlink', shlib)
 
-    def _check_shared_library(self, pkg, pkgfile_path, path):
+    def _check_shared_library(self, pkg, pkgfile):
         """
         Various checks for the shared library.
 
@@ -314,26 +353,37 @@ class BinariesCheck(AbstractCheck):
 
         soname = self.readelf_parser.dynamic_section_info.soname
         if not soname:
-            self.output.add_info('W', pkg, 'no-soname', path)
+            self.output.add_info('W', pkg, 'no-soname', pkgfile.name)
         else:
             if not self.validso_regex.search(soname):
-                self.output.add_info('E', pkg, 'invalid-soname', path, soname)
+                self.output.add_info('E', pkg, 'invalid-soname', pkgfile.name, soname)
             else:
-                self._check_soname_symlink(pkg, path, soname)
+                self._check_soname_symlink(pkg, pkgfile.name, soname)
 
                 # check if the major version of the library is in the package
-                # name
-                res = self.soversion_regex.search(soname)
-                if res:
-                    soversion = res.group(1) or res.group(2)
-                    if soversion and soversion not in pkg.name:
-                        self.output.add_info('E', pkg, 'shlib-policy-name-error', soversion)
+                # name (check only for lib* packages)
+                # for now skip all HPC libraries (#901)
+                if pkg.name.startswith('lib') and not pkgfile.name.startswith(self.hpc_locations):
+                    # SLPP is defined here: https://en.opensuse.org/openSUSE:Shared_library_packaging_policy#Package_naming
+                    # Example:
+                    # SONAME = libgame2-1.9.so.10.0.0
+                    # expected package name: libgame2-1_9-10_0_0
+                    res = self.soversion_regex.search(soname)
+                    if res:
+                        parts = [x.replace('.', '_') for x in (res.group('pkgversion'), res.group('soversion')) if x]
+                        soversion = '-'.join(parts)
+                        pkgname = pkg.name
+                        if '.' in pkgname:
+                            pkgname = pkgname[:pkgname.rindex('.')]
+                        if soversion and not pkgname.endswith(soversion):
+                            self.output.add_info('E', pkg, 'shlib-policy-name-error',
+                                                 f'SONAME: {soname} ({pkgfile.name}), expected package suffix: {soversion}')
 
         # check if the object code in the library is compiled with PIC
         if self.readelf_parser.dynamic_section_info['TEXTREL']:
-            self.output.add_info('E', pkg, 'shlib-with-non-pic-code', path)
+            self.output.add_info('E', pkg, 'shlib-with-non-pic-code', pkgfile.name)
 
-    def _check_dependency(self, pkg, pkgfile_path, path):
+    def _check_dependency(self, pkg, pkgfile):
         """
         FIXME Add test coverage.
         """
@@ -344,15 +394,20 @@ class BinariesCheck(AbstractCheck):
         # following issues are errors for shared libs and warnings for executables
         if not self.is_dynamically_linked:
             return
+
+        # Skip python packages
+        if self.python_module_regex.fullmatch(pkgfile.name):
+            return
+
         if not self.is_archive and not self.readelf_parser.is_debug:
             info_type = 'E' if self.readelf_parser.is_shlib else 'W'
             for symbol in self.ldd_parser.undefined_symbols:
-                self.output.add_info(info_type, pkg, 'undefined-non-weak-symbol', path, symbol)
+                self.output.add_info(info_type, pkg, 'undefined-non-weak-symbol', pkgfile.name, symbol)
             for dependency in self.ldd_parser.unused_dependencies:
                 self.output.add_info(info_type, pkg, 'unused-direct-shlib-dependency',
-                                     path, dependency)
+                                     pkgfile.name, dependency)
 
-    def _check_library_dependency_location(self, pkg, pkgfile_path, path):
+    def _check_library_dependency_location(self, pkg, pkgfile):
         """
         FIXME Add test coverage.
         """
@@ -362,17 +417,17 @@ class BinariesCheck(AbstractCheck):
         if not self.is_archive:
             for dependency in self.ldd_parser.dependencies:
                 if dependency.startswith('/opt/'):
-                    self.output.add_info('E', pkg, 'linked-against-opt-library', path, dependency)
+                    self.output.add_info('E', pkg, 'linked-against-opt-library', pkgfile.name, dependency)
                     break
 
         nonusr = ('/bin', '/lib', '/sbin')
-        if path.startswith(nonusr):
+        if pkgfile.name.startswith(nonusr):
             for dependency in self.ldd_parser.dependencies:
                 if dependency.startswith('/usr/'):
-                    self.output.add_info('W', pkg, 'linked-against-usr-library', path, dependency)
+                    self.output.add_info('W', pkg, 'linked-against-usr-library', pkgfile.name, dependency)
                     break
 
-    def _check_security_functions(self, pkg, pkgfile_path, path):
+    def _check_security_functions(self, pkg, pkgfile):
         setgid = any(self.readelf_parser.symbol_table_info.get_functions_for_regex(self.setgid_call_regex))
         setuid = any(self.readelf_parser.symbol_table_info.get_functions_for_regex(self.setuid_call_regex))
         setgroups = any(self.readelf_parser.symbol_table_info.get_functions_for_regex(self.setgroups_call_regex))
@@ -380,50 +435,49 @@ class BinariesCheck(AbstractCheck):
         gethostbyname = any(self.readelf_parser.symbol_table_info.get_functions_for_regex(self.gethostbyname_call_regex))
 
         if setgid and setuid and not setgroups:
-            self.output.add_info('E', pkg, 'missing-call-to-setgroups-before-setuid', path)
+            is_uid = stat.S_ISUID & pkgfile.mode
+            self.output.add_info('W' if is_uid else 'E', pkg, 'missing-call-to-setgroups-before-setuid', pkgfile.name)
 
         if mktemp:
-            self.output.add_info('E', pkg, 'call-to-mktemp', path)
+            self.output.add_info('E', pkg, 'call-to-mktemp', pkgfile.name)
 
         if gethostbyname:
-            self.output.add_info('W', pkg, 'binary-or-shlib-calls-gethostbyname', path)
+            self.output.add_info('W', pkg, 'binary-or-shlib-calls-gethostbyname', pkgfile.name)
 
-    def _check_rpath(self, pkg, pkgfile_path, path):
-        for runpath in self.readelf_parser.dynamic_section_info.runpath:
-            if runpath in self.system_lib_paths or not self.usr_lib_regex.search(runpath):
-                self.output.add_info('E', pkg, 'binary-or-shlib-defines-rpath', path, runpath)
-                return
+    def _check_rpath(self, pkg, pkgfile):
+        for runpaths in self.readelf_parser.dynamic_section_info.runpaths:
+            for runpath in runpaths.split(':'):
+                if self.rpath_origin in runpath:
+                    runpath = runpath.replace(self.rpath_origin, str(Path(pkgfile.name).parent))
+                    runpath = str(Path(runpath).resolve())
+                if not runpath.startswith(self.system_lib_paths) and not self.usr_lib_regex.search(runpath):
+                    self.output.add_info('E', pkg, 'binary-or-shlib-defines-rpath', pkgfile.name, f'(RUNPATH: {runpaths})')
+                    return
 
-    def _check_library_dependency(self, pkg, pkgfile_path, path):
-        if self.is_archive:
-            return
-        if path.startswith('/lib/modules/'):
+    def _check_library_dependency(self, pkg, pkgfile):
+        if (self.is_archive or
+                any(pkgfile.name.startswith(p) for p in KERNEL_MODULES_PATHS) or
+                self.python_module_regex.fullmatch(pkgfile.name)):
             return
 
         dyn_section = self.readelf_parser.dynamic_section_info
         if not len(dyn_section.needed) and not (dyn_section.soname and
                                                 self.ldso_soname_regex.search(dyn_section.soname)):
-            if self.is_shobj:
-                msg = 'shared-library-without-dependency-information'
-            else:
-                msg = 'statically-linked-binary'
-            self.output.add_info('E', pkg, msg, path)
+            if not self.is_shobj:
+                self.output.add_info('E', pkg, 'statically-linked-binary', pkgfile.name)
         else:
             # linked against libc ?
-            if 'libc.' not in dyn_section.runpath and \
+            if 'libc.' not in dyn_section.runpaths and \
                (not dyn_section.soname or
                 ('libc.' not in dyn_section.soname and
                  not self.ldso_soname_regex.search(dyn_section.soname))):
                 for lib in dyn_section.needed:
                     if 'libc.' in lib:
                         return
-                if self.is_shobj:
-                    msg = 'library-not-linked-against-libc'
-                else:
-                    msg = 'program-not-linked-against-libc'
-                self.output.add_info('W', pkg, msg, path)
+                if not self.is_shobj:
+                    self.output.add_info('W', pkg, 'program-not-linked-against-libc', pkgfile.name)
 
-    def _check_forbidden_functions(self, pkg, pkgfile_path, path):
+    def _check_forbidden_functions(self, pkg, pkgfile):
         forbidden_functions = self.config.configuration['WarnOnFunction']
         if forbidden_functions:
             for name, func in forbidden_functions.items():
@@ -443,10 +497,10 @@ class BinariesCheck(AbstractCheck):
         if not forbidden_calls:
             return
 
-        strings_parser = StringsParser(pkgfile_path)
+        strings_parser = StringsParser(pkgfile.path)
         failed_reason = strings_parser.parsing_failed_reason
         if failed_reason:
-            self.output.add_info('E', pkg, 'strings-failed', path, failed_reason)
+            self.output.add_info('E', pkg, 'strings-failed', pkgfile.name, failed_reason)
             return
 
         forbidden_functions_filtered = []
@@ -457,20 +511,19 @@ class BinariesCheck(AbstractCheck):
                 forbidden_functions_filtered.append(fn)
                 continue
 
-            waiver = any(map(lambda string: f['waiver_regex'].search(string), strings_parser.strings))
+            # See: https://github.com/PyCQA/flake8-bugbear/issues/269
+            waiver = any(f['waiver_regex'].search(string) for string in strings_parser.strings)
             if not waiver:
                 forbidden_functions_filtered.append(fn)
 
         for fn in forbidden_functions_filtered:
-            self.output.add_info('W', pkg, fn, path, forbidden_functions[fn]['f_name'])
+            self.output.add_info('W', pkg, fn, pkgfile.name, forbidden_functions[fn]['f_name'])
 
-    def _check_executable_shlib(self, pkg, pkgfile_path, path):
-        if not self.is_exec and self.readelf_parser.is_shlib:
-            interp = [h for h in self.readelf_parser.program_header_info.headers if h.name == 'INTERP']
-            if interp:
-                self.output.add_info('E', pkg, 'shared-library-not-executable', path)
+    def _check_executable_shlib(self, pkg, pkgfile):
+        if not (pkgfile.mode & stat.S_IEXEC) and self.readelf_parser.is_shlib:
+            self.output.add_info('E', pkg, 'shared-library-not-executable', pkgfile.name)
 
-    def _check_optflags(self, pkg, pkgfile_path, path):
+    def _check_optflags(self, pkg, pkgfile):
         if self.is_archive:
             return
 
@@ -484,21 +537,38 @@ class BinariesCheck(AbstractCheck):
             missing = [mo for mo in mandatory_optflags if mo not in tokens]
             forbidden = [f for f in forbidden_optflags if f in tokens]
             if missing:
-                self.output.add_info('W', pkg, 'missing-mandatory-optflags', path, ' '.join(missing))
+                self.output.add_info('W', pkg, 'missing-mandatory-optflags', pkgfile.name, ' '.join(missing))
             if forbidden:
-                self.output.add_info('E', pkg, 'forbidden-optflags', path, ' '.join(forbidden))
+                self.output.add_info('E', pkg, 'forbidden-optflags', pkgfile.name, ' '.join(forbidden))
 
-    def _is_standard_archive(self, pkg, pkgfile_path, path):
+    def _check_hash_sections(self, pkg, pkgfile):
+        if not self.readelf_parser.is_shlib:
+            return
+
+        for elf_file in self.readelf_parser.section_info.elf_files:
+            needle = {'.hash', '.gnu.hash'}
+            for section in elf_file:
+                if not needle:
+                    break
+                if section.name in needle:
+                    needle.remove(section.name)
+
+            if '.hash' in needle:
+                self.output.add_info('E', pkg, 'missing-hash-section', pkgfile.name)
+            if '.gnu.hash' in needle:
+                self.output.add_info('W', pkg, 'missing-gnu-hash-section', pkgfile.name)
+
+    def _is_standard_archive(self, pkg, pkgfile):
         # skip Klee bytecode archives
-        if pkgfile_path.endswith('.bca'):
+        if pkgfile.path.endswith('.bca'):
             return False
 
         # return false for e.g. Rust or Go packages that are archives
         # but files in the archive are not an ELF container
-        ar_parser = ArParser(pkgfile_path)
+        ar_parser = ArParser(pkgfile.path)
         failed_reason = ar_parser.parsing_failed_reason
         if failed_reason:
-            self.output.add_info('E', pkg, 'ar-failed', path, failed_reason)
+            self.output.add_info('E', pkg, 'ar-failed', pkgfile.name, failed_reason)
             return False
 
         needles = ('__.PKGDEF', '_go_.o', 'lib.rmeta')
@@ -510,36 +580,42 @@ class BinariesCheck(AbstractCheck):
         self.is_archive = 'current ar archive' in magic
         self.is_dynamically_linked = 'dynamically linked' in magic
         self.is_pie_exec = 'pie executable' in magic
+        self.is_nonstandard_archive = False
 
-    def run_elf_checks(self, pkg, pkgfile_path, path):
-        if self.is_archive and not self._is_standard_archive(pkg, pkgfile_path, path):
+    def run_elf_checks(self, pkg, pkgfile):
+        if self.is_archive and not self._is_standard_archive(pkg, pkgfile):
+            self.is_nonstandard_archive = True
             return
 
-        self.readelf_parser = ReadelfParser(pkgfile_path, path)
+        self.readelf_parser = ReadelfParser(pkgfile.path, pkgfile.name)
         failed_reason = self.readelf_parser.parsing_failed_reason()
         if failed_reason:
-            self.output.add_info('E', pkg, 'readelf-failed', path, failed_reason)
+            self.output.add_info('E', pkg, 'readelf-failed', pkgfile.name, failed_reason)
             return
 
         if not self.is_archive:
             if self.is_dynamically_linked:
-                is_installed_pkg = isinstance(pkg, InstalledPkg) or isinstance(pkg, FakePkg)
-                self.ldd_parser = LddParser(pkgfile_path, path, is_installed_pkg)
+                is_installed_pkg = isinstance(pkg, (InstalledPkg, FakePkg))
+                self.ldd_parser = LddParser(pkgfile.path, pkgfile.name, is_installed_pkg)
                 failed_reason = self.ldd_parser.parsing_failed_reason
                 if failed_reason:
-                    self.output.add_info('E', pkg, 'ldd-failed', path, failed_reason)
+                    self.output.add_info('E', pkg, 'ldd-failed', pkgfile.name, failed_reason)
                     return
 
-            self.objdump_parser = ObjdumpParser(pkgfile_path, path)
-            failed_reason = self.objdump_parser.parsing_failed_reason
-            if failed_reason:
-                self.output.add_info('E', pkg, 'objdump-failed', path, failed_reason)
-                return
+            if (self.config.configuration['MandatoryOptflags'] or
+                    self.config.configuration['ForbiddenOptflags']):
+                self.objdump_parser = ObjdumpParser(pkgfile.path, pkgfile.name)
+                failed_reason = self.objdump_parser.parsing_failed_reason
+                if failed_reason:
+                    self.output.add_info('E', pkg, 'objdump-failed', pkgfile.name, failed_reason)
+                    return
 
+        # NOTE: the speed benefit of the ThreadPoolExecutor is limited due to
+        # Global Interpreter Lock (GIL).
         with concurrent.futures.ThreadPoolExecutor() as executor:
             futures = []
             for fn in self.check_functions:
-                futures.append(executor.submit(fn, pkg, pkgfile_path, path))
+                futures.append(executor.submit(fn, pkg, pkgfile))
             concurrent.futures.wait(futures)
             for future in futures:
                 err = future.exception()
@@ -548,7 +624,6 @@ class BinariesCheck(AbstractCheck):
 
     def check_binary(self, pkg):
         exec_files = []
-        multi_pkg = False
         pkg_has_lib = False
         pkg_has_binary = False
         pkg_has_binary_in_usrlib = False
@@ -558,7 +633,6 @@ class BinariesCheck(AbstractCheck):
         #  go through the all files, run files checks and collect data that are
         #  needed later
         for fname, pkgfile in pkg.files.items():
-
             # Common tests first
             self._check_libtool_wrapper(pkg, fname, pkgfile)
             self._check_invalid_la_file(pkg, fname)
@@ -576,17 +650,23 @@ class BinariesCheck(AbstractCheck):
 
             # find out if we have a file in /usr/lib64/ directory (needed later
             # for the package checks)
-            if stat.S_ISREG(pkgfile.mode) and \
-                    (fname.startswith('/usr/lib64') or
-                     fname.startswith('/lib64')):
+            if fname.startswith(('/usr/lib64', '/lib64')):
                 pkg_has_file_in_lib64 = True
 
             # skip the rest of the tests for non-binaries
             # binary files only from here on
             is_ocaml_native = 'Objective caml native' in pkgfile.magic
             is_lua_bytecode = 'Lua bytecode' in pkgfile.magic
-            if not (pkgfile.magic.startswith('ELF ') or 'current ar archive'
-                    in pkgfile.magic or is_ocaml_native or is_lua_bytecode):
+            #  eBPF binaries are arch independent
+            # https://github.com/rpm-software-management/rpmlint/issues/1193
+            is_ebpf = 'eBPF' in pkgfile.magic
+            # Look for ELF in the file magic to check if it's really a binary
+            # file
+            is_elf = self.elf_regex.match(pkgfile.magic) and not is_ebpf
+
+            if not (is_elf or
+                    'current ar archive' in pkgfile.magic or
+                    is_ocaml_native or is_lua_bytecode):
                 continue
 
             self.checked_files += 1
@@ -623,7 +703,10 @@ class BinariesCheck(AbstractCheck):
             self._detect_attributes(pkgfile.magic)
 
             # run ELF checks
-            self.run_elf_checks(pkg, pkgfile.path, fname)
+            self.run_elf_checks(pkg, pkgfile)
+
+            if self.is_nonstandard_archive:
+                continue
 
             # inspect binary file
             is_shlib = self.readelf_parser.is_shlib
@@ -648,19 +731,11 @@ class BinariesCheck(AbstractCheck):
 
                 self._check_non_pie(pkg, fname)
 
-        # find out if we have a multi-package
-        srpm = pkg[rpm.RPMTAG_SOURCERPM]
-        if srpm:
-            srcname = self.srcname_regex.search(srpm)
-            if srcname:
-                multi_pkg = (pkg.name != srcname.group(1))
-
         # run checks for the whole package
         # it uses data collected in the previous for-cycle
         self._check_exec_in_library(pkg, pkg_has_lib, exec_files)
         self._check_non_versioned(pkg, pkg_has_lib, exec_files)
-        self._check_no_binary(pkg, pkg_has_binary, multi_pkg,
-                              pkg_has_file_in_lib64)
+        self._check_no_binary(pkg, pkg_has_binary, pkg_has_file_in_lib64)
         self._check_noarch_with_lib64(pkg, pkg_has_file_in_lib64)
         self._check_only_non_binary_in_usrlib(pkg, pkg_has_usrlib_file,
                                               pkg_has_binary_in_usrlib)
