@@ -1,28 +1,32 @@
 import bz2
 from collections import namedtuple
+import contextlib
 import gzip
+import hashlib
+import io
 import lzma
+import mmap
 import os
-from pathlib import Path
+from pathlib import Path, PurePath
 import re
 from shlex import quote
+import shutil
 import stat
 import subprocess
 import tempfile
+import time
 from urllib.parse import urljoin
 
 try:
     import magic
-    # TODO: magic.MAGIC_COMPRESS when PkgFile gets decompress support.
-    _magic = magic.open(magic.MAGIC_NONE)
-    _ = _magic.descriptor  # magic >= 5.05 needed
-    _magic.load()
+    has_magic = True
 except ImportError:
-    _magic = None
+    has_magic = False
 import rpm
-from rpmlint.helpers import byte_to_string, ENGLISH_ENVIROMENT, print_warning
+from rpmlint.helpers import (byte_to_string, ENGLISH_ENVIRONMENT,
+                             print_warning, pushd)
 from rpmlint.pkgfile import PkgFile
-import zstd
+import zstandard as zstd
 
 
 DepInfo = namedtuple('DepInfo', ('name', 'flags', 'version'))
@@ -111,22 +115,21 @@ def is_utf8(fname):
 def is_utf8_bytestr(s):
     """Returns True whether the given text is UTF-8.
     Due to changes in rpm, needs to handle both bytes and unicode."""
+    if not isinstance(s, (bytes, str)):
+        unexpected = type(s).__name__
+        raise TypeError(f'Expected str/bytes, not {unexpected}')
+
     try:
-        if hasattr(s, 'decode'):
+        if isinstance(s, bytes):
             s.decode('utf-8')
-        elif hasattr(s, 'encode'):
-            s.encode('utf-8')
-        else:
-            unexpected = type(s).__name__
-            raise TypeError(
-                'Expected str/unicode/bytes, not {}'.format(unexpected))
     except UnicodeError:
         return False
+
     return True
 
 
 def has_forbidden_controlchars(val):
-    if isinstance(val, str) or isinstance(val, bytes):
+    if isinstance(val, (str, bytes)):
         string = val
         if isinstance(val, bytes):
             val = memoryview(val)
@@ -135,7 +138,7 @@ def has_forbidden_controlchars(val):
                 c = ord(c)
             if c < 32 and (c not in (9, 10, 13)):
                 return string
-    if isinstance(val, tuple) or isinstance(val, list):
+    if isinstance(val, (tuple, list)):
         for item in val:
             return has_forbidden_controlchars(item)
     return False
@@ -197,33 +200,25 @@ def rangeCompare(reqtuple, provtuple):
     if rc >= 1:
         if reqf in ['GT', 'GE', 4, 12]:
             return 1
-        if reqf in ['EQ', 8]:
-            if f in ['LE', 10, 'LT', 2]:
-                return 1
-        if reqf in ['LE', 'LT', 'EQ', 10, 2, 8]:
-            if f in ['LE', 'LT', 10, 2]:
-                return 1
+        if reqf in ['EQ', 8] and f in ['LE', 10, 'LT', 2]:
+            return 1
+        if reqf in ['LE', 'LT', 'EQ', 10, 2, 8] and f in ['LE', 'LT', 10, 2]:
+            return 1
 
     if rc == 0:
-        if reqf in ['GT', 4]:
-            if f in ['GT', 'GE', 4, 12]:
-                return 1
-        if reqf in ['GE', 12]:
-            if f in ['GT', 'GE', 'EQ', 'LE', 4, 12, 8, 10]:
-                return 1
-        if reqf in ['EQ', 8]:
-            if f in ['EQ', 'GE', 'LE', 8, 12, 10]:
-                return 1
-        if reqf in ['LE', 10]:
-            if f in ['EQ', 'LE', 'LT', 'GE', 8, 10, 2, 12]:
-                return 1
-        if reqf in ['LT', 2]:
-            if f in ['LE', 'LT', 10, 2]:
-                return 1
+        if reqf in ['GT', 4] and f in ['GT', 'GE', 4, 12]:
+            return 1
+        if reqf in ['GE', 12] and f in ['GT', 'GE', 'EQ', 'LE', 4, 12, 8, 10]:
+            return 1
+        if reqf in ['EQ', 8] and f in ['EQ', 'GE', 'LE', 8, 12, 10]:
+            return 1
+        if reqf in ['LE', 10] and f in ['EQ', 'LE', 'LT', 'GE', 8, 10, 2, 12]:
+            return 1
+        if reqf in ['LT', 2] and f in ['LE', 'LT', 10, 2]:
+            return 1
     if rc <= -1:
-        if reqf in ['GT', 'GE', 'EQ', 4, 12, 8]:
-            if f in ['GT', 'GE', 4, 12]:
-                return 1
+        if reqf in ['GT', 'GE', 'EQ', 4, 12, 8] and f in ['GT', 'GE', 4, 12]:
+            return 1
         if reqf in ['LE', 'LT', 10, 2]:
             return 1
 #                if rc >= 1:
@@ -243,17 +238,16 @@ def rangeCompare(reqtuple, provtuple):
 def formatRequire(name, flags, evr):
     s = name
 
-    if flags:
-        if flags & (rpm.RPMSENSE_LESS | rpm.RPMSENSE_GREATER |
-                    rpm.RPMSENSE_EQUAL):
-            s = s + ' '
-            if flags & rpm.RPMSENSE_LESS:
-                s = s + '<'
-            if flags & rpm.RPMSENSE_GREATER:
-                s = s + '>'
-            if flags & rpm.RPMSENSE_EQUAL:
-                s = s + '='
-            s = '%s %s' % (s, versionToString(evr))
+    if flags and flags & (rpm.RPMSENSE_LESS | rpm.RPMSENSE_GREATER |
+                          rpm.RPMSENSE_EQUAL):
+        s = s + ' '
+        if flags & rpm.RPMSENSE_LESS:
+            s = s + '<'
+        if flags & rpm.RPMSENSE_GREATER:
+            s = s + '>'
+        if flags & rpm.RPMSENSE_EQUAL:
+            s = s + '='
+        s = f'{s} {versionToString(evr)}'
     return s
 
 
@@ -278,11 +272,9 @@ def stringToVersion(verstring):
     epoch = None
     i = verstring.find(':')
     if i != -1:
-        try:
-            epoch = str(int(verstring[:i]))
-        except ValueError:
+        with contextlib.suppress(ValueError):
             # garbage in epoch, ignore it
-            pass
+            epoch = int(verstring[:i])
     i += 1
     j = verstring.find('-', i)
     if j != -1:
@@ -365,271 +357,55 @@ def parse_deps(line):
     return prcos
 
 
+def _get_magic_libmagic(path):
+    return magic.detect_from_filename(path).name
+
+
+def _get_magic_python_magic(path):
+    return magic.from_file(path)
+
+
 def get_magic(path):
-    # file() method evaluates every file twice with python2,
-    # use descriptor() method instead
+    # python-magic & libmagic compatibility code
+    # https://github.com/ahupp/python-magic/blob/master/COMPAT.md
+    detect_magic = _get_magic_python_magic
+    if not hasattr(magic, 'from_file'):
+        # libmagic python bindings
+        detect_magic = _get_magic_libmagic
+
     try:
-        fd = os.open(path, os.O_RDONLY)
-        magic = byte_to_string(_magic.descriptor(fd))
-        os.close(fd)
-        return magic
-    except OSError:
+        return detect_magic(path)
+    except (ValueError, FileNotFoundError):
         return ''
 
 
 # classes representing package
 
-class AbstractPkg(object):
-
+class AbstractPkg:
     def cleanup(self):
         pass
 
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        self.cleanup()
-
-
-class Pkg(AbstractPkg):
-
-    _magic_from_compressed_re = re.compile(r'\([^)]+\s+compressed\s+data\b')
-
-    def __init__(self, filename, dirname, header=None, is_source=False, extracted=False, verbose=False):
-        self.filename = filename
-        self.extracted = extracted
-        self.dirname = self.dir_name(dirname, verbose)
-        self.current_linenum = None
-
-        self._req_names = -1
-
-        if header:
-            self.header = header
-            self.is_source = is_source
-        else:
-            # Create a package object from the file name
-            ts = rpm.TransactionSet()
-            # Don't check signatures here...
-            ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES)
-            fd = os.open(filename, os.O_RDONLY)
-            try:
-                self.header = ts.hdrFromFdno(fd)
-            finally:
-                os.close(fd)
-            self.is_source = not self.header[rpm.RPMTAG_SOURCERPM]
-
-        self.name = self[rpm.RPMTAG_NAME]
-
-        (self.requires, self.prereq, self.provides, self.conflicts,
-         self.obsoletes, self.recommends, self.suggests, self.enhances,
-         self.supplements) = self._gatherDepInfo()
-
-        self.req_names = [x[0] for x in self.requires + self.prereq]
-
-        self.files = self._gatherFilesInfo()
-        self.config_files = [x.name for x in self.files.values() if x.is_config]
-        self.doc_files = [x.name for x in self.files.values() if x.is_doc]
-        self.ghost_files = [x.name for x in self.files.values() if x.is_ghost]
-        self.noreplace_files = [x.name for x in self.files.values() if x.is_noreplace]
-        self.missingok_files = [x.name for x in self.files.values() if x.is_missingok]
-
-        if self.is_no_source:
-            self.arch = 'nosrc'
-        elif self.is_source:
-            self.arch = 'src'
-        else:
-            self.arch = self.header.format('%{ARCH}')
-
-    # Return true if the package is a nosource package.
-    # NoSource files are ghosts in source packages.
-    @property
-    def is_no_source(self):
-        return self.is_source and self.ghost_files
-
-    # access the tags like an array
-    def __getitem__(self, key):
-        try:
-            val = self.header[key]
-        except KeyError:
-            val = []
-        if val == []:
-            return None
-        else:
-            # Note that text tags we want to try decoding for real in TagsCheck
-            # such as summary, description and changelog are not here.
-            if key in (rpm.RPMTAG_NAME, rpm.RPMTAG_VERSION, rpm.RPMTAG_RELEASE,
-                       rpm.RPMTAG_ARCH, rpm.RPMTAG_GROUP, rpm.RPMTAG_BUILDHOST,
-                       rpm.RPMTAG_LICENSE, rpm.RPMTAG_HEADERI18NTABLE,
-                       rpm.RPMTAG_PACKAGER, rpm.RPMTAG_SOURCERPM,
-                       rpm.RPMTAG_DISTRIBUTION, rpm.RPMTAG_VENDOR) \
-            or key in (x[0] for x in SCRIPT_TAGS) \
-            or key in (x[1] for x in SCRIPT_TAGS):
-                val = byte_to_string(val)
-                if key == rpm.RPMTAG_GROUP and val == 'Unspecified':
-                    val = None
-            return val
-
-    # return the name of the directory where the package is extracted
-    def dirName(self):
-        return self.dirname
-
-    def dir_name(self, dirname, verbose):
-        return self._extract(dirname, verbose)
-
-    # extract rpm contents
-    def _extract(self, dirname, verbose):
-        if not Path(dirname).is_dir():
-            print_warning('Unable to access dir %s' % dirname)
-        else:
-            dirname = dirname if dirname != '/' else None
-            self.__tmpdir = tempfile.TemporaryDirectory(
-                prefix='rpmlint.%s.' % Path(self.filename).name, dir=dirname
-            )
-            dirname = self.__tmpdir.name
-            # TODO: sequence based command invocation
-            # TODO: warn some way if this fails (e.g. rpm2cpio not installed)
-            command_str = \
-                'rpm2cpio %(f)s | cpio -id -D %(d)s ; chmod -R +rX %(d)s' % \
-                {'f': quote(str(self.filename)), 'd': quote(dirname)}
-            stderr = None if verbose else subprocess.DEVNULL
-            subprocess.check_output(command_str, shell=True, env=ENGLISH_ENVIROMENT,
-                                    stderr=stderr)
-            self.extracted = True
-        return dirname
-
-    def checkSignature(self):
-        ret = subprocess.run(('rpm', '-Kv', self.filename),
-                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                             env=ENGLISH_ENVIROMENT)
-        text = ret.stdout.decode()
-        if text.endswith('\n'):
-            text = text[:-1]
-        return ret.returncode, text
-
-    # remove the extracted files from the package
-    def cleanup(self):
-        if self.extracted and self.dirname:
-            self.__tmpdir.cleanup()
-
-    def grep(self, regex, filename):
-        """Grep regex from a file, return matching line numbers."""
-        ret = []
-        lineno = 0
-        try:
-            with open(Path(self.dirName() or '/', filename.lstrip('/'))) as in_file:
-                for line in in_file:
-                    lineno += 1
-                    if regex.search(line):
-                        ret.append(str(lineno))
-                        break
-        except Exception:
-            pass
-        return ret
-
-    def langtag(self, tag, lang):
-        """Get value of tag in the given language."""
-        # LANGUAGE trumps other env vars per GNU gettext docs, see also #166
-        orig = os.environ.get('LANGUAGE')
-        os.environ['LANGUAGE'] = lang
-        ret = self[tag]
-        if orig is not None:
-            os.environ['LANGUAGE'] = orig
-        return ret
-
-    # extract information about the files
-    def _gatherFilesInfo(self):
-
-        ret = {}
-        flags = self.header[rpm.RPMTAG_FILEFLAGS]
-        modes = self.header[rpm.RPMTAG_FILEMODES]
-        users = self.header[rpm.RPMTAG_FILEUSERNAME]
-        groups = self.header[rpm.RPMTAG_FILEGROUPNAME]
-        links = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILELINKTOS]]
-        sizes = self.header[rpm.RPMTAG_FILESIZES]
-        md5s = self.header[rpm.RPMTAG_FILEMD5S]
-        mtimes = self.header[rpm.RPMTAG_FILEMTIMES]
-        rdevs = self.header[rpm.RPMTAG_FILERDEVS]
-        langs = self.header[rpm.RPMTAG_FILELANGS]
-        inodes = self.header[rpm.RPMTAG_FILEINODES]
-        requires = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILEREQUIRE]]
-        provides = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILEPROVIDE]]
-        files = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILENAMES]]
-        magics = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILECLASS]]
-        try:  # rpm >= 4.7.0
-            filecaps = self.header[rpm.RPMTAG_FILECAPS]
-        except AttributeError:
-            filecaps = None
-
-        # rpm-python < 4.6 does not return a list for this (or FILEDEVICES,
-        # FWIW) for packages containing exactly one file
-        if not isinstance(inodes, list):
-            inodes = [inodes]
-
-        if files:
-            for idx, file in enumerate(files):
-                pkgfile = PkgFile(file)
-                pkgfile.path = os.path.normpath(os.path.join(
-                    self.dirName() or '/', pkgfile.name.lstrip('/')))
-                pkgfile.flags = flags[idx]
-                pkgfile.mode = modes[idx]
-                pkgfile.user = byte_to_string(users[idx])
-                pkgfile.group = byte_to_string(groups[idx])
-                pkgfile.linkto = links[idx] and os.path.normpath(links[idx])
-                pkgfile.size = sizes[idx]
-                pkgfile.md5 = md5s[idx]
-                pkgfile.mtime = mtimes[idx]
-                pkgfile.rdev = rdevs[idx]
-                pkgfile.inode = inodes[idx]
-                pkgfile.requires = parse_deps(requires[idx])
-                pkgfile.provides = parse_deps(provides[idx])
-                pkgfile.lang = byte_to_string(langs[idx])
-                pkgfile.magic = magics[idx]
-                if not pkgfile.magic:
-                    if stat.S_ISDIR(pkgfile.mode):
-                        pkgfile.magic = 'directory'
-                    elif stat.S_ISLNK(pkgfile.mode):
-                        pkgfile.magic = "symbolic link to `%s'" % pkgfile.linkto
-                    elif not pkgfile.size:
-                        pkgfile.magic = 'empty'
-                if (not pkgfile.magic and
-                        not pkgfile.is_ghost and _magic):
-                    pkgfile.magic = get_magic(pkgfile.path)
-                if pkgfile.magic is None:
-                    pkgfile.magic = ''
-                elif Pkg._magic_from_compressed_re.search(pkgfile.magic):
-                    # Discard magic from inside compressed files ('file -z')
-                    # until PkgFile gets decompression support.  We may get
-                    # such magic strings from package headers already now;
-                    # for example Fedora's rpmbuild as of F-11's 4.7.1 is
-                    # patched so it generates them.
-                    pkgfile.magic = ''
-                if filecaps:
-                    pkgfile.filecaps = byte_to_string(filecaps[idx])
-                ret[pkgfile.name] = pkgfile
-        return ret
-
-    def readlink(self, pkgfile):
-        """
-        Resolve symlinks for the given PkgFile, return the dereferenced
-        PkgFile if it is found in this package, None if not.
-        """
-        result = pkgfile
-        while result and result.linkto:
-            linkpath = urljoin(result.name, result.linkto)
-            linkpath = os.path.normpath(linkpath)
-            result = self.files.get(linkpath)
-        return result
-
-    def check_versioned_dep(self, name, version):
-        # try to match name%_isa as well (e.g. 'foo(x86-64)', 'foo(x86-32)')
-        name_re = re.compile(r'^%s(\(\w+-\d+\))?$' % re.escape(name))
-        for d in self.requires + self.prereq:
-            if name_re.match(d[0]):
-                if d[1] & rpm.RPMSENSE_EQUAL != rpm.RPMSENSE_EQUAL \
-                        or d[2][1] != version:
-                    return False
-                return True
-        return False
+    def _calc_magic(self, pkgfile):
+        magic = pkgfile.magic
+        if not magic:
+            if stat.S_ISDIR(pkgfile.mode):
+                magic = 'directory'
+            elif stat.S_ISLNK(pkgfile.mode):
+                magic = "symbolic link to `%s'" % pkgfile.linkto
+            elif not pkgfile.size:
+                magic = 'empty'
+        if not magic and not pkgfile.is_ghost and has_magic:
+            start = time.monotonic()
+            magic = get_magic(pkgfile.path)
+            self.timers['libmagic'] += time.monotonic() - start
+        if magic is None or Pkg._magic_from_compressed_re.search(magic):
+            # Discard magic from inside compressed files ('file -z')
+            # until PkgFile gets decompression support.  We may get
+            # such magic strings from package headers already now;
+            # for example Fedora's rpmbuild as of F-11's 4.7.1 is
+            # patched so it generates them.
+            magic = ''
+        return magic
 
     # internal function to gather dependency info used by the above ones
     def _gather_aux(self, header, xs, nametag, flagstag, versiontag,
@@ -648,7 +424,7 @@ class Pkg(AbstractPkg):
                     xs.append(DepInfo(name, flags[loop], evr))
         return xs, prereq
 
-    def _gatherDepInfo(self):
+    def _gather_dep_info(self):
         _requires = []
         _prereq = []
         _provides = []
@@ -712,8 +488,246 @@ class Pkg(AbstractPkg):
             prog = ''.join(prog)
         return prog
 
+    def __enter__(self):
+        return self
 
-def getInstalledPkgs(name):
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.cleanup()
+
+
+class Pkg(AbstractPkg):
+    _magic_from_compressed_re = re.compile(r'\([^)]+\s+compressed\s+data\b')
+
+    def __init__(self, filename, dirname, header=None, is_source=False, extracted=False, verbose=False):
+        self.filename = filename
+        self.extracted = extracted
+
+        # record decompression and extraction time
+        start = time.monotonic()
+        self.dirname = self._extract_rpm(dirname, verbose)
+        self.timers = {'ExtractRpm': time.monotonic() - start, 'libmagic': 0}
+        self.current_linenum = None
+
+        self._req_names = -1
+
+        if header:
+            self.header = header
+            self.is_source = is_source
+        else:
+            # Create a package object from the file name
+            ts = rpm.TransactionSet()
+            # Don't check signatures here...
+            ts.setVSFlags(rpm._RPMVSF_NOSIGNATURES)
+            fd = os.open(filename, os.O_RDONLY)
+            try:
+                self.header = ts.hdrFromFdno(fd)
+            finally:
+                os.close(fd)
+            self.is_source = not self.header[rpm.RPMTAG_SOURCERPM]
+
+        self.name = self[rpm.RPMTAG_NAME]
+
+        (self.requires, self.prereq, self.provides, self.conflicts,
+         self.obsoletes, self.recommends, self.suggests, self.enhances,
+         self.supplements) = self._gather_dep_info()
+
+        self.req_names = [x[0] for x in self.requires + self.prereq]
+
+        self.files = self._gather_files_info()
+        self.config_files = [x.name for x in self.files.values() if x.is_config]
+        self.doc_files = [x.name for x in self.files.values() if x.is_doc]
+        self.ghost_files = [x.name for x in self.files.values() if x.is_ghost]
+        self.noreplace_files = [x.name for x in self.files.values() if x.is_noreplace]
+        self.missingok_files = [x.name for x in self.files.values() if x.is_missingok]
+
+        if self.is_no_source:
+            self.arch = 'nosrc'
+        elif self.is_source:
+            self.arch = 'src'
+        else:
+            self.arch = self.header.format('%{ARCH}')
+
+    # Return true if the package is a nosource package.
+    # NoSource files are ghosts in source packages.
+    @property
+    def is_no_source(self):
+        return self.is_source and self.ghost_files
+
+    # access the tags like an array
+    def __getitem__(self, key):
+        try:
+            val = self.header[key]
+        except KeyError:
+            val = []
+        if val == []:
+            return None
+        else:
+            # Note that text tags we want to try decoding for real in TagsCheck
+            # such as summary, description and changelog are not here.
+            if key in (rpm.RPMTAG_NAME, rpm.RPMTAG_VERSION, rpm.RPMTAG_RELEASE,
+                       rpm.RPMTAG_ARCH, rpm.RPMTAG_GROUP, rpm.RPMTAG_BUILDHOST,
+                       rpm.RPMTAG_LICENSE, rpm.RPMTAG_HEADERI18NTABLE,
+                       rpm.RPMTAG_PACKAGER, rpm.RPMTAG_SOURCERPM,
+                       rpm.RPMTAG_DISTRIBUTION, rpm.RPMTAG_VENDOR) \
+            or key in (x[0] for x in SCRIPT_TAGS) \
+            or key in (x[1] for x in SCRIPT_TAGS):
+                val = byte_to_string(val)
+                if key == rpm.RPMTAG_GROUP and val == 'Unspecified':
+                    val = None
+            return val
+
+    # return the name of the directory where the package is extracted
+    def dir_name(self):
+        return self.dirname
+
+    def _extract_rpm(self, dirname, verbose):
+        if not Path(dirname).is_dir():
+            print_warning('Unable to access dir %s' % dirname)
+        elif dirname == '/':
+            # it is an InstalledPkg
+            pass
+        else:
+            self.__tmpdir = tempfile.TemporaryDirectory(
+                prefix='rpmlint.%s.' % Path(self.filename).name, dir=dirname
+            )
+            dirname = self.__tmpdir.name
+
+            # BusyBox' cpio does not support '-D' argument and the only safe
+            # usage is doing chdir before invocation.
+            filename = Path(self.filename).resolve()
+            with pushd(dirname):
+                stderr = None if verbose else subprocess.DEVNULL
+                if shutil.which('rpm2archive'):
+                    with open(filename, 'rb') as rpm_data:
+                        subprocess.check_output('rpm2archive - | tar -xz && chmod -R +rX .', shell=True, env=ENGLISH_ENVIRONMENT,
+                                                stderr=stderr, stdin=rpm_data)
+                else:
+                    command_str = f'rpm2cpio {quote(str(filename))} | cpio -id ; chmod -R +rX .'
+                    subprocess.check_output(command_str, shell=True, env=ENGLISH_ENVIRONMENT, stderr=stderr)
+            self.extracted = True
+        return dirname
+
+    def check_signature(self):
+        ret = subprocess.run(('rpm', '-Kv', self.filename),
+                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                             env=ENGLISH_ENVIRONMENT, text=True)
+        text = ret.stdout
+        if text.endswith('\n'):
+            text = text[:-1]
+        return ret.returncode, text
+
+    # remove the extracted files from the package
+    def cleanup(self):
+        if self.extracted and self.dirname:
+            self.__tmpdir.cleanup()
+
+    def grep(self, regex, filename):
+        """Grep regex from a file, return first matching line number (starting with 1)."""
+        data = self.read_with_mmap(filename)
+        match = regex.search(data)
+        if match:
+            return data.count('\n', 0, match.start()) + 1
+        else:
+            return None
+
+    def read_with_mmap(self, filename):
+        """Mmap a file, return it's content decoded."""
+        try:
+            with open(Path(self.dir_name() or '/', filename.lstrip('/'))) as in_file:
+                return mmap.mmap(in_file.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ).read().decode()
+        except Exception:
+            return ''
+
+    def langtag(self, tag, lang):
+        """Get value of tag in the given language."""
+        # LANGUAGE trumps other env vars per GNU gettext docs, see also #166
+        orig = os.environ.get('LANGUAGE')
+        os.environ['LANGUAGE'] = lang
+        ret = self[tag]
+        if orig is not None:
+            os.environ['LANGUAGE'] = orig
+        return ret
+
+    # extract information about the files
+    def _gather_files_info(self):
+        ret = {}
+        flags = self.header[rpm.RPMTAG_FILEFLAGS]
+        modes = self.header[rpm.RPMTAG_FILEMODES]
+        users = self.header[rpm.RPMTAG_FILEUSERNAME]
+        groups = self.header[rpm.RPMTAG_FILEGROUPNAME]
+        links = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILELINKTOS]]
+        sizes = self.header[rpm.RPMTAG_FILESIZES]
+        if len(sizes) != len(flags):
+            sizes = self.header[rpm.RPMTAG_LONGFILESIZES]
+        md5s = self.header[rpm.RPMTAG_FILEMD5S]
+        mtimes = self.header[rpm.RPMTAG_FILEMTIMES]
+        rdevs = self.header[rpm.RPMTAG_FILERDEVS]
+        langs = self.header[rpm.RPMTAG_FILELANGS]
+        inodes = self.header[rpm.RPMTAG_FILEINODES]
+        requires = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILEREQUIRE]]
+        provides = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILEPROVIDE]]
+        files = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILENAMES]]
+        magics = [byte_to_string(x) for x in self.header[rpm.RPMTAG_FILECLASS]]
+        try:  # rpm >= 4.7.0
+            filecaps = self.header[rpm.RPMTAG_FILECAPS]
+        except AttributeError:
+            filecaps = None
+
+        # rpm-python < 4.6 does not return a list for this (or FILEDEVICES,
+        # FWIW) for packages containing exactly one file
+        if not isinstance(inodes, list):
+            inodes = [inodes]
+
+        if files:
+            for idx, file in enumerate(files):
+                pkgfile = PkgFile(file)
+                pkgfile.path = os.path.normpath(os.path.join(
+                    self.dir_name() or '/', pkgfile.name.lstrip('/')))
+                pkgfile.flags = flags[idx]
+                pkgfile.mode = modes[idx]
+                pkgfile.user = byte_to_string(users[idx])
+                pkgfile.group = byte_to_string(groups[idx])
+                pkgfile.linkto = links[idx] and os.path.normpath(links[idx])
+                pkgfile.size = sizes[idx]
+                pkgfile.md5 = md5s[idx]
+                pkgfile.mtime = mtimes[idx]
+                pkgfile.rdev = rdevs[idx]
+                pkgfile.inode = inodes[idx]
+                pkgfile.requires = parse_deps(requires[idx])
+                pkgfile.provides = parse_deps(provides[idx])
+                pkgfile.lang = byte_to_string(langs[idx])
+                pkgfile.magic = magics[idx]
+                pkgfile.magic = self._calc_magic(pkgfile)
+                if filecaps:
+                    pkgfile.filecaps = byte_to_string(filecaps[idx])
+                ret[pkgfile.name] = pkgfile
+        return ret
+
+    def readlink(self, pkgfile):
+        """
+        Resolve symlinks for the given PkgFile, return the dereferenced
+        PkgFile if it is found in this package, None if not.
+        """
+        result = pkgfile
+        while result and result.linkto:
+            linkpath = urljoin(result.name, result.linkto)
+            linkpath = os.path.normpath(linkpath)
+            result = self.files.get(linkpath)
+        return result
+
+    def check_versioned_dep(self, name, version):
+        # try to match name%_isa as well (e.g. 'foo(x86-64)', 'foo(x86-32)')
+        name_re = re.compile(r'^%s(\(\w+-\d+\))?$' % re.escape(name))
+        for d in self.requires + self.prereq:
+            if name_re.match(d[0]):
+                if d[1] & rpm.RPMSENSE_EQUAL != rpm.RPMSENSE_EQUAL \
+                        or d[2][1] != version:
+                    return False
+                return True
+        return False
+
+
+def get_installed_pkgs(name):
     """Get list of installed package objects by name."""
 
     pkgs = []
@@ -752,14 +766,50 @@ class InstalledPkg(Pkg):
     def cleanup(self):
         pass
 
-    def checkSignature(self):
+    def check_signature(self):
         return (0, 'fake: pgp md5 OK')
 
 
+class FakeHeader(dict):
+    def sprintf(self, expr):
+        """
+        Replaces expressions like %{} with actual package
+        """
+
+        tagre = re.compile(r'%{([^}]*)}')
+        for tag in tagre.findall(expr):
+            expr = expr.replace(f'%{tag}', self[f'RPMTAG_{tag}'])
+        return expr
+
+    def __missing__(self, key):
+        try:
+            key = getattr(rpm, key)
+        except (TypeError, KeyError):
+            raise KeyError
+
+        if key not in self:
+            raise KeyError
+
+        return self[key]
+
+
 # Class to provide an API to a 'fake' package, eg. for specfile-only checks
 class FakePkg(AbstractPkg):
+    _autoheaders = [
+        'requires',
+        'conflicts',
+        'provides',
+        'obsoletes',
+        'recommends',
+        'suggests',
+        'enhances',
+        'supplements',
+    ]
+
     def __init__(self, name, is_source=False):
+        self.timers = {'ExtractRpm': 0, 'libmagic': 0}
         self.name = str(name)
+        self.filename = f'{name}.rpm'
         self.arch = None
         self.current_linenum = None
         self.dirname = None
@@ -769,20 +819,149 @@ class FakePkg(AbstractPkg):
         self.files = {}
         self.ghost_files = {}
 
-    def add_file_with_content(self, name, content, **flags):
+        # header is a dictionary to mock rpm metadata
+        self.header = FakeHeader()
+        for i in self._autoheaders:
+            # the header name wihtout the ending 's'
+            tagname = i[:-1].upper()
+            self.header[getattr(rpm, f'RPMTAG_{tagname}NAME')] = []
+            self.header[getattr(rpm, f'RPMTAG_{tagname}FLAGS')] = []
+            self.header[getattr(rpm, f'RPMTAG_{tagname}VERSION')] = []
+        self.header[rpm.RPMTAG_FILENAMES] = []
+
+    def add_file(self, path, name):
+        pkgfile = PkgFile(name)
+        pkgfile.path = path
+        self.files[name] = pkgfile
+        return pkgfile
+
+    def _mock_file(self, path, attrs):
+        metadata = None
+        if attrs.get('create_dirs', False):
+            for i in PurePath(path).parents[:attrs.get('include_dirs', -1)]:
+                self.add_dir(str(i))
+        metadata = attrs.get('metadata', None)
+
+        if attrs.get('is_dir', False):
+            self.add_dir(path, metadata=metadata)
+            return
+
+        content = ''
+        if 'content-path' in attrs:
+            content = open(attrs['content-path'], 'rb')
+        elif 'content' in attrs:
+            content = attrs['content']
+
+        if 'linkto' in attrs:
+            self.add_symlink_to(path, attrs['linkto'])
+        else:
+            self.add_file_with_content(path, content, metadata=metadata)
+        self.header[rpm.RPMTAG_FILENAMES].append(path)
+
+        if 'content-path' in attrs:
+            content.close()
+
+    def create_files(self, files):
+        """
+        This is a helper method to create files(real files); not PkgFile
+        objects.
+        """
+
+        # files can be just a list
+        if isinstance(files, list) or isinstance(files, tuple):
+            for path in files:
+                self._mock_file(path, {})
+        # list of files with attributes and content
+        elif isinstance(files, dict):
+            for path, file in files.items():
+                self._mock_file(path, file)
+
+    def add_dir(self, path, metadata=None):
+        pkgdir = PkgFile(path)
+        pkgdir.magic = 'directory'
+
+        path = os.path.join(self.dir_name(), path.lstrip('/'))
+        os.makedirs(Path(path), exist_ok=True)
+
+        pkgdir.path = path
+        self.files[path] = pkgdir
+
+        if metadata:
+            for k, v in metadata.items():
+                setattr(pkgdir, k, v)
+
+        return pkgdir
+
+    def add_file_with_content(self, name, content, metadata=None, **flags):
         """
         Add file to the FakePkg and fill the file with provided
         string content.
         """
-        basename = name.replace(os.path.sep, '_')
-        path = os.path.join(self.dirName(), basename)
-        with open(path, 'w') as out:
-            out.write(content)
-            pkg_file = PkgFile(name)
-            pkg_file.path = path
-            for key, value in flags.items():
-                setattr(pkg_file, key, value)
-            self.files[name] = pkg_file
+        path = os.path.join(self.dir_name(), name.lstrip('/'))
+        pkg_file = PkgFile(name)
+        pkg_file.path = path
+        pkg_file.mode = stat.S_IFREG | 0o0644
+        pkg_file.user = 'root'
+        pkg_file.group = 'root'
+        self.files[name] = pkg_file
+
+        # create files in filesystem
+        os.makedirs(Path(path).parent, exist_ok=True)
+
+        if isinstance(content, str):
+            content = content.encode('utf-8', errors='ignore')
+
+        with open(Path(path), 'wb') as out:
+            # file like content
+            if isinstance(content, io.IOBase):
+                shutil.copyfileobj(content, out)
+            else:
+                out.write(content)
+
+        # Generating md5 hash values for real files:
+        pkg_file.md5 = self.md5_checksum(Path(path))
+        pkg_file.size = os.path.getsize(Path(path))
+        pkg_file.inode = os.stat(Path(path)).st_ino
+        pkg_file.magic = self._calc_magic(pkg_file)
+
+        if metadata:
+            for k, v in metadata.items():
+                setattr(pkg_file, k, v)
+        for key, value in flags.items():
+            setattr(pkg_file, key, value)
+
+    def initiate_files_base_data(self):
+        """ This method is called after adding metadata of each file """
+        self.config_files = [x.name for x in self.files.values() if x.is_config]
+        self.doc_files = [x.name for x in self.files.values() if x.is_doc]
+        self.ghost_files = [x.name for x in self.files.values() if x.is_ghost]
+        self.noreplace_files = [x.name for x in self.files.values() if x.is_noreplace]
+        self.missingok_files = [x.name for x in self.files.values() if x.is_missingok]
+
+    def add_header(self, header):
+        for k, v in header.items():
+            if k in self._autoheaders:
+                # the header name wihtout the ending 's'
+                tagname = k[:-1].upper()
+                for i in v:
+                    name, flags, version = parse_deps(i)[0]
+                    version = f'{version[0]}:{version[1]}-{version[2]}'
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}NAME')].append(name)
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}FLAGS')].append(flags)
+                    self.header[getattr(rpm, f'RPMTAG_{tagname}VERSION')].append(version)
+                continue
+
+            key = getattr(rpm, f'RPMTAG_{k}'.upper())
+            self.header[key] = v
+
+            if key == rpm.RPMTAG_ARCH:
+                self.arch = v
+
+        (self.requires, self.prereq, self.provides, self.conflicts,
+         self.obsoletes, self.recommends, self.suggests, self.enhances,
+         self.supplements) = self._gather_dep_info()
+
+        self.req_names = [x[0] for x in self.requires + self.prereq]
 
     def add_symlink_to(self, name, target):
         """
@@ -793,18 +972,31 @@ class FakePkg(AbstractPkg):
         pkg_file = PkgFile(name)
         pkg_file.mode = stat.S_IFLNK
         pkg_file.linkto = target
+        pkg_file.user = 'root'
+        pkg_file.group = 'root'
         self.files[name] = pkg_file
 
     def readlink(self, pkgfile):
         # HACK: reuse the real Pkg's logic
         return Pkg.readlink(self, pkgfile)
 
-    def dirName(self):
+    def dir_name(self):
         if not self.dirname:
             self.__tmpdir = tempfile.TemporaryDirectory(prefix='rpmlint.%s.' % Path(self.name).name)
             self.dirname = self.__tmpdir.name
         return self.dirname
 
+    def md5_checksum(self, file_name):
+        md5_hash = hashlib.md5()
+        with open(file_name, 'rb') as f:
+            for byte_block in iter(lambda: f.read(4096), b''):
+                md5_hash.update(byte_block)
+        return md5_hash.hexdigest()
+
     def cleanup(self):
         if self.dirname:
             self.__tmpdir.cleanup()
+
+    # access the tags like an array
+    def __getitem__(self, key):
+        return self.header.get(key, None)
