@@ -12,19 +12,37 @@ def speccheck():
     CONFIG.info = True
     output = Filter(CONFIG)
     test = SpecCheck(CONFIG, output)
-    return output, test
+    yield output, test
 
 
-def test_check_include(tmpdir, speccheck):
+@pytest.fixture
+def output(speccheck):
+    output, _test = speccheck
+    yield output
+
+
+@pytest.fixture
+def test(speccheck):
+    _output, test = speccheck
+    yield test
+
+
+def test_check_include(tmp_path, speccheck):
     output, test = speccheck
-    test.check_source(get_tested_package('source/CheckInclude', tmpdir))
+    test.check_source(get_tested_package('source/CheckInclude', tmp_path))
     out = output.print_results(output.results)
     assert "specfile-error can't parse specfile" not in out
     assert 'no-buildroot-tag' in out
     assert 'E: specfile-error error: query of specfile' not in out
 
 
-@pytest.mark.parametrize('package', ['spec/SpecCheck2', 'spec/SpecCheck3'])
+@pytest.mark.parametrize('package', [
+    'spec/SpecCheck2',
+    'spec/SpecCheck3',
+    'spec/SpecCheckPatch',
+    'spec/intltool',
+    'spec/ghc',
+])
 def test_patch_not_applied(package, speccheck):
     output, test = speccheck
     pkg = get_tested_spec_package(package)
@@ -67,19 +85,19 @@ def test_forbidden_controlchars_found(package, speccheck):
 
 
 @pytest.mark.parametrize('package', ['source/no-spec-file'])
-def test_check_no_spec_file(tmpdir, package, speccheck):
+def test_check_no_spec_file(tmp_path, package, speccheck):
     """Test if spec file is not found inside RPM metadata."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: no-spec-file' in out
 
 
 @pytest.mark.parametrize('package', ['source/CheckInclude'])
-def test_check_no_spec_file_not_applied(tmpdir, package, speccheck):
+def test_check_no_spec_file_not_applied(tmp_path, package, speccheck):
     """Test if there is no spec file inside RPM metadata."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: no-spec-file' not in out
 
@@ -105,19 +123,19 @@ def test_check_non_utf8_spec_file_not_applied(package, speccheck):
 
 
 @pytest.mark.parametrize('package', ['source/invalid-spec-name'])
-def test_check_invalid_spec_name(tmpdir, package, speccheck):
+def test_check_invalid_spec_name(tmp_path, package, speccheck):
     """Test if specfile name does not matches the ('Name: ') tag."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: invalid-spec-name' in out
 
 
 @pytest.mark.parametrize('package', ['source/CheckInclude'])
-def test_check_invalid_spec_name_not_applied(tmpdir, package, speccheck):
+def test_check_invalid_spec_name_not_applied(tmp_path, package, speccheck):
     """Test if specfile has specfile name as ('Name: ') tag."""
     output, test = speccheck
-    test.check(get_tested_package(package, tmpdir))
+    test.check(get_tested_package(package, tmp_path))
     out = output.print_results(output.results)
     assert 'E: invalid-spec-name' not in out
 
@@ -172,6 +190,17 @@ def test_check_rpm_buildroot_usage_not_applied(package, speccheck):
     assert 'E: rpm-buildroot-usage' not in out
 
 
+@pytest.mark.parametrize('package', ['spec/rpm-buildroot-usage-shell-var'])
+def test_check_rpm_buildroot_usage_shell_var(package, speccheck):
+    """Test detection of $RPM_BUILD_ROOT shell variable in %prep/%build"""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'E: rpm-buildroot-usage %prep' not in out
+    assert out.count('E: rpm-buildroot-usage %build') == 2
+
+
 @pytest.mark.parametrize('package', ['spec/make-check-outside-check-section'])
 def test_check_make_check_outside_check_section(package, speccheck):
     """Test if specfile has `make check` outside %check."""
@@ -716,6 +745,16 @@ def test_check_macro_in_changelog(package, speccheck):
     assert 'W: macro-in-%changelog' in out
 
 
+@pytest.mark.parametrize('package', ['spec/macro-in-changelog-autochangelog'])
+def test_check_autochangelog(package, speccheck):
+    """Test usage of %autochangelog macro."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: macro-in-%changelog' not in out
+
+
 @pytest.mark.parametrize('package', ['spec/SpecCheck2'])
 def test_check_macro_in_changelog_not_found(package, speccheck):
     """Test if specfile has macro in %changelog
@@ -857,6 +896,7 @@ def test_check_no_essential_section(package, speccheck):
     assert 'W: no-%prep-section' in out
     assert 'W: no-%install-section' in out
     assert 'W: no-%build-section' in out
+    assert 'W: no-%check-section' in out
 
 
 @pytest.mark.parametrize('package', ['spec/SpecCheck2'])
@@ -872,6 +912,7 @@ def test_check_no_essential_section_not_applied(package, speccheck):
     assert 'W: no-%install-section' not in out
     assert 'E: superfluous-%clean-section' in out
     assert 'W: no-%build-section' not in out
+    assert 'W: no-%check-section' not in out
 
 
 @pytest.mark.parametrize('package', ['spec/more-than-one-%changelog'])
@@ -1030,6 +1071,34 @@ def test_check_patch_not_applied(package, speccheck):
     assert 'W: patch-not-applied' in out
 
 
+@pytest.mark.parametrize('package', [
+    'spec/SpecCheck',
+    'spec/%ifarch-applied-patch',
+    'spec/prereq_use',
+    'spec/mixed-use-of-spaces-and-tabs',
+])
+def test_check_patch_rpm_420(package, speccheck):
+    """Test if specfile uses %patchN."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'E: patch-macro-old-format' in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/intltool',
+    'spec/ghc',
+    'spec/SpecCheckPatch',
+])
+def test_check_patch_rpm_not_420(package, speccheck):
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'E: patch-macro-old-format' not in out
+
+
 @pytest.mark.parametrize('package', ['spec/mixed-use-of-spaces-and-tabs'])
 def test_check_patch_not_found(package, speccheck):
     """Test if specfile have all patch applied by %autopatch."""
@@ -1071,3 +1140,85 @@ def test_check_invalid_url_not_applied(package, speccheck):
     test.check_spec(pkg)
     out = output.print_results(output.results)
     assert 'W: invalid-url' not in out
+
+
+@pytest.mark.parametrize('package', ['spec/python-setup-test'])
+def test_python_setup_test(package, speccheck):
+    """Test if specfile has deprecated use of 'setup.py test'."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-setup-test' in out
+
+
+@pytest.mark.parametrize('package', ['spec/python-module-def'])
+def test_python_module_definition(package, speccheck):
+    """Test if python_module macro is defined in the spec file."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-module-def' in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/SpecCheck',
+    'spec/SpecCheck2',
+    'spec/SpecCheck3',
+    'spec/SpecCheck4',
+])
+def test_python_module_definition_not_present(package, speccheck):
+    """Test if python_module macro warning is not shown if the macro is not
+       defined in the spec file.
+    """
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-module-def' not in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/python-sitelib-glob',
+    'spec/python-sitearch-glob',
+])
+def test_python_sitelib_glob(package, speccheck):
+    """Test if %{python_sitelib}/* is present in %files section."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-sitelib-glob-in-files' in out
+
+
+@pytest.mark.parametrize('package', ['spec/python-sitelib'])
+def test_python_sitelib(package, speccheck):
+    """Test if %{python_sitelib}/* is present in %files section."""
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'W: python-sitelib-glob-in-files' not in out
+
+
+@pytest.mark.parametrize('package', [
+    'spec/null-char-last',
+    'spec/null-char-first',
+])
+def test_null_char(package, speccheck):
+    output, test = speccheck
+    pkg = get_tested_spec_package(package)
+    test.check_spec(pkg)
+    out = output.print_results(output.results)
+    assert 'forbidden-controlchar-found' in out
+
+
+@pytest.mark.parametrize('package', [
+    get_tested_spec_package('spec/MacroInComment'),
+])
+def test_special_comments(package, output, test):
+    test.check_spec(package)
+    out = output.print_results(output.results)
+    assert 'W: macro-in-comment %configure' in out
+    assert 'W: macro-in-comment %{name}' not in out
